from .default_urls import *

__title__ = 'ska.contrib.django.ska.urls'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)



import datetime
from importlib import import_module
import time

from six.moves.urllib.parse import quote

from .defaults import SIGNATURE_LIFETIME

__title__ = 'ska.helpers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_callback_func',
    'dict_keys',
    'dict_to_ordered_list',
    'sorted_urlencode',
    'extract_signed_data',
    'make_valid_until',
)


def get_callback_func(func, fail_silently=True):
    """Take a string and try to extract a function from it.

    :param mixed func: If `callable` is given, return as is. If `string`
        is given, try to extract the function from the string given and
        return.
    :param fail_silently: bool
    :return callable: Returns `callable` if what's extracted is callable or
        None otherwise.
    """
    if callable(func):
        return func
    elif isinstance(func, str):
        try:
            module_path, class_name = func.rsplit('.', 1)
        except ValueError as err:
            if not fail_silently:
                raise ImportError(
                    "%s doesn't look like a module path" % func)

        module = import_module(module_path)

        try:
            return getattr(module, class_name)
        except AttributeError as err:
            if not fail_silently:
                raise ImportError(
                    'Module "%s" does not define a "%s" attribute/class' % (
                        module_path, class_name)
                    )


def dict_keys(data, return_string=False):
    """Get sorted keys from dictionary given.

    If ``return_string`` argument is set to True, returns keys joined by
    commas.

    :param dict data:
    :param bool return_string:
    """
    keys = list(data.keys())
    keys.sort()

    if return_string:
        return ','.join(keys)

    return keys


def dict_to_ordered_list(data):
    """Get extra as ordered list.

    Actually, I'm not sure whether I should or should not be using
    ``OrderedDict`` here.

    :param dict data:
    :return list:
    """
    items = list(data.items())
    items.sort()
    return items


def sorted_urlencode(data, quoted=True):
    """Similar to built-in ``urlencode``, but always puts data in a sorted
    constant way that stays the same between various python versions.
    """
    _sorted = ["{0}={1}".format(k, v) for k, v in dict_to_ordered_list(data)]
    res = '&'.join(_sorted)
    if quoted:
        res = quote(res)
    return res


def extract_signed_data(data, extra):
    """Filters out non-white-listed items from the ``extra`` dictionary given.

    :param dict data:
    :param list extra:
    :return dict:
    """
    extracted_extra = dict(data)
    for key, value in data.items():
        if key not in extra:
            extracted_extra.pop(key)

    return extracted_extra


def make_valid_until(lifetime=SIGNATURE_LIFETIME):
    """Make valid until.

    :param int lifetime:
    :return datetime.datetime"""
    return time.mktime(
        (
            datetime.datetime.now() +
            datetime.timedelta(seconds=lifetime)
        ).timetuple()
    )



from __future__ import absolute_import

from django import template

from ..... import sign_url as ska_sign_url
from .....defaults import (
    SIGNATURE_LIFETIME,
    DEFAULT_URL_SUFFIX,
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_EXTRA_PARAM,
)
from .....signatures import Signature
from ..settings import SECRET_KEY

__title__ = 'ska.contrib.django.ska.templatetags.ska_tags'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('sign_url',)


register = template.Library()


@register.simple_tag(takes_context=True)
def sign_url(context,
             url='',
             auth_user=None,
             secret_key=SECRET_KEY,
             valid_until=None,
             lifetime=SIGNATURE_LIFETIME,
             suffix=DEFAULT_URL_SUFFIX,
             signature_param=DEFAULT_SIGNATURE_PARAM,
             auth_user_param=DEFAULT_AUTH_USER_PARAM,
             valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
             extra=None,
             extra_param=DEFAULT_EXTRA_PARAM,
             signature_cls=Signature):
    """Sign URL."""
    # The `extra` and `extra_param` are not used at the moment.
    if not auth_user:
        auth_user = context['request'].user.get_username()

    return ska_sign_url(
        auth_user=auth_user,
        secret_key=secret_key,
        valid_until=valid_until,
        lifetime=lifetime,
        url=url,
        suffix=suffix,
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra=extra,
        extra_param=extra_param,
        signature_cls=signature_cls
    )



from six.moves.urllib.parse import urlencode

from .defaults import (
    DEFAULT_URL_SUFFIX,
    DEFAULT_EXTRA_PARAM,
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
)
from .exceptions import InvalidData, ImproperlyConfigured
from .helpers import (
    dict_keys, extract_signed_data as extract_signed_data
)
from .signatures import Signature

__title__ = 'ska.utils'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('RequestHelper',)

# ***************************************************************************
# ***************************************************************************
# **************************** Request helper *******************************
# ***************************************************************************
# ***************************************************************************


class RequestHelper(object):
    """Request helper for easy put/extract of signature params from URLs.

    :param str signature_param:
    :param str auth_user_param:
    :param str valid_until_param:
    :param str extra_param:
    """
    def __init__(self,
                 signature_param=DEFAULT_SIGNATURE_PARAM,
                 auth_user_param=DEFAULT_AUTH_USER_PARAM,
                 valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                 extra_param=DEFAULT_EXTRA_PARAM,
                 signature_cls=Signature):
        """Constructor."""
        self.signature_param = signature_param
        self.auth_user_param = auth_user_param
        self.valid_until_param = valid_until_param
        self.extra_param = extra_param
        self.signature_cls = signature_cls

    def signature_to_url(self, signature, endpoint_url='',
                         suffix=DEFAULT_URL_SUFFIX):
        """URL encodes the signature params.

        :param ska.Signature signature:
        :param str endpoint_url:
        :param str suffix: Suffix to add after the ``endpoint_url`` and before
            the appended signature params.
        :return str:

        :example:

        Required imports.

        >>> from ska import Signature, RequestHelper

        Generate signature.

        >>> signature = Signature.generate_signature(
        >>>     auth_user='user',
        >>>     secret_key='your-secret-key'
        >>> )

        Create a request helper.

        >>> request_helper = RequestHelper(
        >>>     signature_param='signature',
        >>>     auth_user_param='auth_user',
        >>>     valid_until_param='valid_until'
        >>> )

        Appending signature params to the endpoint URL.

        >>> url = request_helper.signature_to_url(
        >>>     signature=signature,
        >>>     endpoint_url='http://e.com/api/'
        >>> )
        http://e.com/api/?valid_until=1378045287.0&auth_user=user&signature=YlZpLFsjUKBalL4x5trhkeEgqE8%3D
        """
        params = {
            self.signature_param: signature.signature,
            self.auth_user_param: signature.auth_user,
            self.valid_until_param: signature.valid_until,
            self.extra_param: dict_keys(signature.extra, return_string=True),
        }

        # Make some check that params used do not overlap with names
        # reserved (`auth_user`, `signature`, etc).
        params.update(signature.extra)

        return "{0}{1}{2}".format(endpoint_url, suffix, urlencode(params))

    def signature_to_dict(self, signature):
        """Put signature into a dictionary.

         Dictionary can be used later on to send requests (for example, a POST
         request) to the server.

        :param ska.Signature signature:
        :return dict:

        :example:

        Required imports.

        >>> from ska import Signature, RequestHelper

        Generate signature.

        >>> signature = Signature.generate_signature(
        >>>     auth_user='user',
        >>>     secret_key='your-secret-key'
        >>> )

        Create a request helper.

        >>> request_helper = RequestHelper(
        >>>     signature_param='signature',
        >>>     auth_user_param='auth_user',
        >>>     valid_until_param='valid_until'
        >>> )

        Appending signature params to the endpoint URL.

        >>> signed_dict = request_helper.signature_to_dict(
        >>>     signature=signature
        >>> )
        {
            'signature': 'YlZpLFsjUKBalL4x5trhkeEgqE8=',
            'auth_user': 'user',
            'valid_until': '1378045287.0'
        }
        """
        data = {
            self.signature_param: signature.signature,
            self.auth_user_param: signature.auth_user,
            self.valid_until_param: signature.valid_until,
            self.extra_param: dict_keys(signature.extra, return_string=True),
        }

        data.update(signature.extra)

        return data

    def validate_request_data(self, data, secret_key):
        """Validate the request data.

        :param dict data:
        :param str secret_key:
        :return ska.SignatureValidationResult:

        :example:
        If your imaginary `HttpRequest` object has `GET` property (dict),
        then you would validate the request data as follows.

        Create a `RequestHelper` object with param names expected.

        Required imports.

        >>> from ska import RequestHelper

        Create a request helper.

        >>> request_helper = RequestHelper(
        >>>     signature_param='signature',
        >>>     auth_user_param='auth_user',
        >>>     valid_until_param='valid_until'
        >>> )

        Validate the request data.

        >>> validation_result = request_helper.validate_request_data(
        >>>     data=request.GET,
        >>>     secret_key='your-secret-key'
        >>> )
        """
        signature = data.get(self.signature_param, '')
        auth_user = data.get(self.auth_user_param, '')
        valid_until = data.get(self.valid_until_param, '')

        extra = extract_signed_data(
            data=data,
            extra=data.get(self.extra_param, '').split(',')
        )

        validation_result = self.signature_cls.validate_signature(
            signature=signature,
            auth_user=auth_user,
            secret_key=secret_key,
            valid_until=valid_until,
            return_object=True,
            extra=extra
        )

        return validation_result

    def extract_signed_data(self, data, secret_key=None, validate=False,
                            fail_silently=False):
        """Extract signed data from the request."""
        if validate:
            if not secret_key:
                if fail_silently:
                    return {}
                raise ImproperlyConfigured("You should provide `secret_key` "
                                           "if `validate` is set to True.")
            validation_result = self.validate_request_data(data, secret_key)
            if not validation_result.result:
                if fail_silently:
                    return {}
                raise InvalidData(validation_result.message)

        return extract_signed_data(
            data=data,
            extra=data.get(self.extra_param, '').split(',')
        )



from __future__ import absolute_import

# __title__ = 'ska.contrib.django.ska.apps'
# __author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
# __copyright__ = '2013-2018 Artur Barseghyan'
# __license__ = 'GPL 2.0/LGPL 2.1'
# __all__ = ('default_app_config',)
#
# default_app_config = 'ska.contrib.django.ska.apps.Config'



from __future__ import absolute_import

from django.http import HttpResponseForbidden

__title__ = 'ska.contrib.django.ska.http'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('HttpResponseUnauthorized',)


class HttpResponseUnauthorized(HttpResponseForbidden):
    """HttpResponseUnauthorized.

    http://en.wikipedia.org/wiki/List_of_HTTP_status_codes#4xx_Client_Error
    """
    status_code = 401



from __future__ import absolute_import
import logging

from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import User
from django.db import IntegrityError

from nine import versions

from ..... import Signature, extract_signed_request_data
from .....helpers import get_callback_func
from .....defaults import (
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_EXTRA_PARAM,
)
from .....exceptions import ImproperlyConfigured, InvalidData

from ..models import Signature as SignatureModel
from ..settings import (
    SECRET_KEY,
    USER_GET_CALLBACK,
    USER_CREATE_CALLBACK,
    USER_INFO_CALLBACK,
    DB_STORE_SIGNATURES,
    DB_PERFORM_SIGNATURE_CHECK,
)
from ..utils import get_provider_data

logger = logging.getLogger(__file__)

__title__ = 'ska.contrib.django.ska.backends.constance_backend'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseSkaAuthenticationBackend',)


class BaseSkaAuthenticationBackend(object):
    """Base authentication backend."""

    def get_settings(self):
        """

        :return:
        """
        raise NotImplementedError(
            "You should implement this method in your authentication backend"
        )

    def authenticate(self, request, **kwargs):
        """Authenticate.

        :param django.http.HttpRequest request:
        :return django.contrib.auth.models.User: Instance or None on failure.
        """
        if versions.DJANGO_GTE_1_7:
            request_data = request.GET.dict()
        else:
            request_data = request.REQUEST

        provider_settings = self.get_settings()

        provider_data = get_provider_data(request_data, provider_settings)

        if provider_data:
            secret_key = provider_data['SECRET_KEY']
        else:
            secret_key = SECRET_KEY

        try:
            # If authentication/data validation failed.
            signed_request_data = extract_signed_request_data(
                data=request_data,
                secret_key=secret_key,
                signature_param=DEFAULT_SIGNATURE_PARAM,
                auth_user_param=DEFAULT_AUTH_USER_PARAM,
                valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                extra_param=DEFAULT_EXTRA_PARAM,
                validate=True,
                fail_silently=False
            )
        except (ImproperlyConfigured, InvalidData) as err:
            logger.debug(str(err))
            return None

        # Get the username from request.
        auth_user = request_data.get(DEFAULT_AUTH_USER_PARAM)
        signature = request_data.get(DEFAULT_SIGNATURE_PARAM)
        valid_until = request_data.get(DEFAULT_VALID_UNTIL_PARAM)

        # All other specific data is taken from signed request data
        email = signed_request_data.get('email', '')
        first_name = signed_request_data.get('first_name', '')
        last_name = signed_request_data.get('last_name', '')

        # Storing the signatures to database if set to be so.
        if DB_STORE_SIGNATURES:
            token = SignatureModel(
                auth_user=auth_user,
                signature=signature,
                valid_until=Signature.unix_timestamp_to_date(valid_until)
            )
            try:
                token.save()
            except IntegrityError:
                if DB_PERFORM_SIGNATURE_CHECK:
                    # Token has already been used. Do not authenticate.
                    return None

        # Try to get user. If doesn't exist - create.
        try:
            user = User._default_manager.get(username=auth_user)

            # User get callback
            user_get_callback = provider_data.get(
                'USER_GET_CALLBACK',
                USER_GET_CALLBACK
            )
            if user_get_callback is not None:
                callback_func = get_callback_func(user_get_callback)
                if callback_func:
                    try:
                        callback_func(
                            user,
                            request=request,
                            signed_request_data=signed_request_data
                        )
                    except Exception as err:
                        logger.debug(str(err))

        except User.DoesNotExist:
            user = User._default_manager.create_user(
                username=auth_user,
                email=email,
                password=make_password(password=None),
                first_name=first_name,
                last_name=last_name
            )
            user.save()

            # User create callback
            user_create_callback = provider_data.get(
                'USER_CREATE_CALLBACK',
                USER_CREATE_CALLBACK
            )
            if user_create_callback is not None:
                callback_func = get_callback_func(user_create_callback)
                if callback_func:
                    try:
                        user_create_callback_resp = callback_func(
                            user,
                            request=request,
                            signed_request_data=signed_request_data
                        )
                    except Exception as err:
                        logger.debug(str(err))

        # User info callback
        user_info_callback = provider_data.get(
            'USER_INFO_CALLBACK',
            USER_INFO_CALLBACK
        )
        if user_info_callback is not None:
            callback_func = get_callback_func(user_info_callback)
            if callback_func:
                try:
                    callback_func(
                        user,
                        request=request,
                        signed_request_data=signed_request_data
                    )
                except Exception as err:
                    logger.debug(str(err))

        return user

    def get_user(self, user_id):
        """Get user in the ``django.contrib.auth.models.User`` if exists.

        :param int user_id:
        :return django.contrib.auth.models.User:
        """
        try:
            return User._default_manager.get(pk=user_id)
        except User.DoesNotExist:
            return None



# Generated by Django 2.1.3 on 2018-12-07 20:54

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Signature',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('signature', models.CharField(max_length=255, verbose_name='Signature')),
                ('auth_user', models.CharField(max_length=255, verbose_name='Auth user')),
                ('valid_until', models.DateTimeField(verbose_name='Valid until')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
            ],
            options={
                'verbose_name': 'Token',
                'verbose_name_plural': 'Tokens',
            },
        ),
        migrations.AlterUniqueTogether(
            name='signature',
            unique_together={('signature', 'auth_user', 'valid_until')},
        ),
    ]



import unittest

import shlex
import subprocess

from six.moves.urllib import parse

from ..shortcuts import validate_signed_request_data

__title__ = 'ska.tests.test_commands'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'GenerateSignedUrlTest',
)


class GenerateSignedUrlTest(unittest.TestCase):
    """Tests of `generate_signed_url` module and `ska-sign-url` script."""

    def setUp(self):
        """Set up."""
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.url = 'http://example.com/'

    def test_generate_signed_url(self):
        """Test `generate_signed_url` module.

        :return:
        """
        signed_url = subprocess.check_output(
            shlex.split(
                'ska-sign-url '
                '-au {auth_user} '
                '-sk {secret_key} '
                '--url {url}'.format(
                    auth_user=self.auth_user,
                    secret_key=self.secret_key,
                    url=self.url,
                )
            )
        ).strip()
        # It's necessary to `strip` the value, since in Python 2 there might
        # be a \n added at the end of the string.

        parsed_url = parse.urlparse(str(signed_url))
        parsed_query_params = parse.parse_qs(parsed_url.query)
        data = {
            'signature': parsed_query_params.get('signature')[0],
            'auth_user': parsed_query_params.get('auth_user')[0],
            'valid_until': parsed_query_params.get('valid_until')[0],
        }

        validation_result = validate_signed_request_data(data, self.secret_key)

        self.assertTrue(validation_result.result)



from django.core.management.base import BaseCommand

from ska.contrib.django.ska.utils import purge_signature_data

__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Command',)


class Command(BaseCommand):
    def handle(self, *args, **options):
        """
        Purges old signature data (valid_until < now).
        """
        purge_signature_data()



import unittest
import datetime
from copy import copy

from six import PY3, text_type

from .. import (
    Signature,
    RequestHelper,
    HMACMD5Signature,
    HMACSHA1Signature,
    HMACSHA224Signature,
    HMACSHA256Signature,
    HMACSHA384Signature,
    HMACSHA512Signature,
)
from .. import sign_url, validate_signed_request_data, signature_to_dict
from .. import error_codes
from .base import log_info, timestamp_to_human_readable, parse_url_params

__title__ = 'ska.tests.test_core'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SignatureTest',
    'URLHelperTest',
    'ShortcutsTest',
    'ExtraTest',
)


class SignatureTest(unittest.TestCase):
    """Tests of `ska.Signature` class."""

    def setUp(self):
        """Set up."""
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.signature_classes = (
            HMACMD5Signature,
            HMACSHA1Signature,
            HMACSHA224Signature,
            HMACSHA256Signature,
            HMACSHA384Signature,
            HMACSHA512Signature,
        )

    @log_info
    def __test_01_signature_test(self, signature_cls=Signature):
        """Signature test."""
        flow = []

        flow.append(('Signature class', signature_cls))

        # Generate signature
        sig = signature_cls.generate_signature(
            auth_user=self.auth_user,
            secret_key=self.secret_key
        )

        flow.append(('Valid until used', sig.valid_until))
        flow.append(('Valid until (human readable)',
                     timestamp_to_human_readable(sig.valid_until)))
        flow.append(('Signature generated', sig.signature))
        flow.append(('Signature is expired', sig.is_expired()))

        # Check if not expired
        self.assertTrue(not sig.is_expired())

        # Check if valid
        validation_result = signature_cls.validate_signature(
            signature=sig.signature,
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            valid_until=sig.valid_until,
            return_object=True
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_01_signature_test(self):
        """Signature test."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_01_signature_test(signature_cls=signature_cls)
        return flow

    @log_info
    def __test_02_signature_test_with_positive_time_lapse(
            self, signature_cls=Signature):
        """Signature test with positive time-lapse.

        When signature is made on a host that has a positive (greater) time
        difference with server. In this particular example, the host time is
        5 minutes ahead the server time.
        """
        flow = []

        flow.append(('Signature class', signature_cls))

        datetime_time_lapse = signature_cls.datetime_to_unix_timestamp(
            datetime.datetime.now() + datetime.timedelta(seconds=300)
        )

        flow.append(('Valid until used', datetime_time_lapse))
        flow.append(('Valid until used (human readable)',
                     timestamp_to_human_readable(datetime_time_lapse)))

        # Generate signature
        sig = signature_cls.generate_signature(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            valid_until=datetime_time_lapse
        )

        flow.append(('Signature generated', sig.signature))
        flow.append(('Signature is expired', sig.is_expired()))

        # Check if not expired
        self.assertTrue(not sig.is_expired())

        # Check if valid
        validation_result = signature_cls.validate_signature(
            signature=sig.signature,
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            valid_until=sig.valid_until,
            return_object=True
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_02_signature_test_with_positive_time_lapse(self):
        """Signature test with positive time-lapse.

        When signature is made on a host that has a positive (greater) time
        difference with server. In this particular example, the host time is
        5 minutes ahead the server time.
        """
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_02_signature_test_with_positive_time_lapse(
                signature_cls=signature_cls
            )
        return flow

    @log_info
    def __test_03_signature_test_with_negative_time_lapse(
            self, signature_cls=Signature):
        """Fail test for signature test with negative time-lapse.

        When signature is made on a host that has a negative (less) time
        difference with server. In this particular example, the host time is
        5 minutes behind the server time, which exceeds the signature
        lifetime.
        """
        flow = []

        flow.append(('Signature class', signature_cls))

        datetime_time_lapse = signature_cls.datetime_to_unix_timestamp(
            datetime.datetime.now() - datetime.timedelta(seconds=300)
        )

        flow.append(('Valid until used', datetime_time_lapse))
        flow.append(('Valid until used (human readable)',
                     timestamp_to_human_readable(datetime_time_lapse)))

        # Generate signature
        sig = signature_cls.generate_signature(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            valid_until=datetime_time_lapse
        )

        flow.append(('Signature generated', sig.signature))
        flow.append(('Signature is expired', sig.is_expired()))

        # Check if not expired
        self.assertTrue(sig.is_expired())

        # Check if valid
        validation_result = signature_cls.validate_signature(
            signature=sig.signature,
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            valid_until=sig.valid_until,
            return_object=True
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    def test_03_signature_test_with_negative_time_lapse(self):
        """Fail test. Signature test with negative time-lapse.

        When signature is made on
        a host that has a negative (less) time difference with server. In this
        particular example, the host time is 5 minutes behind the server time,
        which exceeds the signature lifetime.
        """
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_03_signature_test_with_negative_time_lapse(
                signature_cls=signature_cls
            )
        return flow

    @log_info
    def __test_04_fail_signature_test(self, signature_cls=Signature):
        """Fail signature tests."""
        flow = []

        flow.append(('Signature class', signature_cls))

        validation_result = signature_cls.validate_signature(
            signature='EBS6ipiqRLa6TY5vxIvZU30FpnM=',
            auth_user='fakeuser',
            secret_key='fakesecret',
            valid_until=1377997396.0,
            return_object=True
        )

        flow.append(('Valid until used', 1377997396.0))
        flow.append(('Valid until used (human readable)',
                     timestamp_to_human_readable(1377997396.0)))
        flow.append(('Signature generated', 'EBS6ipiqRLa6TY5vxIvZU30FpnM='))
        flow.append(('Signature is expired', True))

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    def test_04_fail_signature_test(self):
        """Fail signature tests."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_04_fail_signature_test(
                signature_cls=signature_cls
            )
        return flow

    @log_info
    def __test_05_fail_signature_test_validation_result_class(
            self, signature_cls=Signature):
        """Fail signature tests for `ValidationResult` class."""
        flow = []

        flow.append(('Signature class', signature_cls))

        validation_result = signature_cls.validate_signature(
            signature='EBS6ipiqRLa6TY5vxIvZU30FpnM=',
            auth_user='fakeuser',
            secret_key='fakesecret',
            valid_until=1377997396.0,
            return_object=True
        )

        if PY3:
            self.assertIsInstance(validation_result.reason, map)
        else:
            self.assertIsInstance(validation_result.reason, list)
        self.assertIsInstance(validation_result.errors, list)
        self.assertIsInstance(validation_result.message, text_type)
        self.assertIsInstance(' '.join(validation_result.reason), text_type)
        self.assertIsInstance(
            ' '.join(map(text_type, validation_result.errors)),
            text_type
        )

        flow.append(validation_result.message)

        self.assertIn(error_codes.INVALID_SIGNATURE,
                      validation_result.errors)
        self.assertIn(error_codes.SIGNATURE_TIMESTAMP_EXPIRED,
                      validation_result.errors)

        return flow

    def test_05_fail_signature_test_validation_result_class(self):
        """Fail signature tests of the `ValidationResult` class."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += \
                self.__test_05_fail_signature_test_validation_result_class(
                    signature_cls=signature_cls
                )
        return flow


class URLHelperTest(unittest.TestCase):
    """Tests of `ska.URLHelper` class."""

    def setUp(self):
        """Set up."""
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.signature_classes = (
            HMACMD5Signature,
            HMACSHA1Signature,
            HMACSHA224Signature,
            HMACSHA256Signature,
            HMACSHA384Signature,
            HMACSHA512Signature,
        )

    @log_info
    def __test_01_signature_to_url(self, signature_cls=Signature):
        """Signature test."""
        flow = []

        flow.append(('Signature class', signature_cls))

        # Generate signature
        signature = signature_cls.generate_signature(
            auth_user=self.auth_user,
            secret_key=self.secret_key
        )

        request_helper = RequestHelper(
            signature_param='signature',
            auth_user_param='auth_user',
            valid_until_param='valid_until',
            signature_cls=signature_cls
        )

        signed_endpoint_url = request_helper.signature_to_url(
            signature=signature,
            endpoint_url='http://dev.example.com/api/'
        )

        flow.append(('URL generated', signed_endpoint_url))

        # Now parsing back the URL params.
        request_data = parse_url_params(signed_endpoint_url)

        validation_result = request_helper.validate_request_data(
            data=request_data,
            secret_key=self.secret_key
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_01_signature_to_url(self):
        """Signature test."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_01_signature_to_url(
                signature_cls=signature_cls
            )
        return flow

    @log_info
    def __test_02_signature_to_url_fail(self, signature_cls=Signature):
        """Signature test. Fail test."""
        flow = []

        flow.append(('Signature class', signature_cls))

        datetime_time_lapse = signature_cls.datetime_to_unix_timestamp(
            datetime.datetime.now() - datetime.timedelta(seconds=300)
        )

        # Generate signature
        signature = signature_cls.generate_signature(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            valid_until=datetime_time_lapse
        )

        request_helper = RequestHelper(
            signature_param='signature',
            auth_user_param='auth_user',
            valid_until_param='valid_until',
            signature_cls=signature_cls
        )

        signed_endpoint_url = request_helper.signature_to_url(
            signature=signature,
            endpoint_url='http://dev.example.com/api/'
        )

        flow.append(('URL generated', signed_endpoint_url))

        # Now parsing back the URL params.
        request_data = parse_url_params(signed_endpoint_url)

        validation_result = request_helper.validate_request_data(
            data=request_data,
            secret_key=self.secret_key
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    def test_02_signature_to_url_fail(self):
        """Signature test. Fail test."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_02_signature_to_url_fail(
                signature_cls=signature_cls
            )
        return flow


class ShortcutsTest(unittest.TestCase):
    """Tests for shortcut functions.

    The following shortcut functions are tested: `sign_url`,
    `signature_to_dict` and `validate_signed_request_data`.
    """

    def setUp(self):
        """Set up."""
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.endpoint_url = 'http://e.com/api/'
        self.signature_classes = (
            HMACMD5Signature,
            HMACSHA1Signature,
            HMACSHA224Signature,
            HMACSHA256Signature,
            HMACSHA384Signature,
            HMACSHA512Signature,
        )

    @log_info
    def __test_sign_url_validate_signed_request_data(
            self, signature_cls=Signature):
        """Tests for `sign_url` & `validate_signed_request_data`."""
        flow = []

        flow.append(('Signature class', signature_cls))

        signed_url = sign_url(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            url=self.endpoint_url
        )

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        validation_result = validate_signed_request_data(
            data=request_data,
            secret_key=self.secret_key
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_01_sign_url_and_validate_signed_request_data(self):
        """Tests for ``sign_url`` & ``validate_signed_request_data``."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_sign_url_validate_signed_request_data(
                signature_cls=signature_cls
            )
        return flow

    @log_info
    def __test_sign_url_validate_signed_request_data_fail(
            self, signature_cls=Signature):
        """Fail tests for `sign_url` & `validate_signed_request_data`."""
        flow = []

        flow.append(('Signature class', signature_cls))

        datetime_time_lapse = signature_cls.datetime_to_unix_timestamp(
            datetime.datetime.now() - datetime.timedelta(seconds=300)
        )

        flow.append(('Valid until used', datetime_time_lapse))
        flow.append(('Valid until used (human readable)',
                     timestamp_to_human_readable(datetime_time_lapse)))

        signed_url = sign_url(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            url=self.endpoint_url,
            valid_until=datetime_time_lapse
        )

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        validation_result = validate_signed_request_data(
            data=request_data,
            secret_key=self.secret_key
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    def test_02_sign_url_and_validate_signed_request_data_fail(self):
        """Fail tests for `sign_url` & `validate_signed_request_data`."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += \
                self.__test_sign_url_validate_signed_request_data_fail(
                    signature_cls=signature_cls
                )
        return flow

    @log_info
    def __test_signature_to_dict_validate_signed_request_data(
            self, signature_cls=Signature):
        """
        Tests for `signature_to_dict` & `validate_signed_request_data`."""
        flow = []

        flow.append(('Signature class', signature_cls))

        signature_dict = signature_to_dict(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            signature_cls=signature_cls
        )

        flow.append(('Dictionary created', signature_dict))

        # Now validate the signature data

        validation_result = validate_signed_request_data(
            data=signature_dict,
            secret_key=self.secret_key,
            signature_cls=signature_cls
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_03_signature_to_dict_and_validate_signed_request_data(self):
        """Tests for `signature_to_dict` & `validate_signed_request_data`."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += \
                self.__test_signature_to_dict_validate_signed_request_data(
                    signature_cls=signature_cls
                )
        return flow


class ExtraTest(unittest.TestCase):
    """Test for extra data."""

    def setUp(self):
        """Set up."""
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.endpoint_url = 'http://e.com/api/'
        self.signature_classes = (
            HMACMD5Signature,
            HMACSHA1Signature,
            HMACSHA224Signature,
            HMACSHA256Signature,
            HMACSHA384Signature,
            HMACSHA512Signature,
        )

    def __get_signed_url(self, signature_cls=Signature):
        """Get signed URL."""
        return sign_url(
            auth_user=self.auth_user,
            secret_key=self.secret_key,
            url=self.endpoint_url,
            extra={
                'provider': 'service1.example.com',
                'first_name': 'John',
                'last_name': 'Doe',
                'email': 'john.doe@mail.example.com',
            },
            signature_cls=signature_cls
        )

    @log_info
    def __test_sign_url_validate_signed_request_data(
            self, signature_cls=Signature):
        """Tests for `sign_url` & `validate_signed_request_data`."""
        flow = []

        flow.append(('Signature class', signature_cls))

        signed_url = self.__get_signed_url(signature_cls=signature_cls)

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        # request_data['extra'] = 'provider,first_name'

        validation_result = validate_signed_request_data(
            data=request_data,
            secret_key=self.secret_key,
            signature_cls=signature_cls
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_01_sign_url_and_validate_signed_request_data(self):
        """Tests for ``sign_url`` and ``validate_signed_request_data``."""
        flow = []
        for signature_cls in self.signature_classes:
            flow += self.__test_sign_url_validate_signed_request_data(
                signature_cls=signature_cls
            )
        return flow

    @log_info
    def __t_sign_url_validate_sgnd_req_data_tamper_extra_keys_rm(
            self, signature_cls=Signature):
        """Fail tests for `sign_url` and `validate_signed_request_data`.

        As well as providing the additional data ``extra`` and data tampering
        ``extra`` keys (remove).
        """
        flow = []

        flow.append(('Signature class', signature_cls))

        signed_url = self.__get_signed_url(signature_cls=signature_cls)

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        # *******************************************************************
        # ************************ Tampering ********************************
        # *******************************************************************
        tampered_request_data = copy(request_data)

        tampered_request_data['extra'] = 'provider,first_name'

        flow.append(('Request data', request_data))
        flow.append(('Tampered request data', tampered_request_data))

        validation_result = validate_signed_request_data(
            data=tampered_request_data,
            secret_key=self.secret_key,
            signature_cls=signature_cls
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    def test_02_sign_url_validate_signed_req_data_tamper_extra_keys_rm(self):
        """Fail tests for `sign_url` and `validate_signed_request_data`.

        As well as providing the additional data `extra` and data tampering
        `extra` keys (remove).
        """
        flow = []
        for signature_cls in self.signature_classes:
            flow += \
                self.__t_sign_url_validate_sgnd_req_data_tamper_extra_keys_rm(
                    signature_cls=signature_cls
                )
        return flow

    @log_info
    def __t_sgn_url_and_vldt_sgnd_req_data_tamper_extra_keys_add(
            self, signature_cls=Signature):
        """Fail tests for `sign_url` and `validate_signed_request_data`.

        As well as providing the additional data ``extra`` and data tampering
        `extra` keys (add).
        """
        flow = []

        flow.append(('Signature class', signature_cls))

        signed_url = self.__get_signed_url(signature_cls=signature_cls)

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        # *******************************************************************
        # ************************* Tampering *******************************
        # *******************************************************************
        tampered_request_data = copy(request_data)

        tampered_request_data['extra'] += ',age'
        tampered_request_data['age'] = 27

        flow.append(('Request data', request_data))
        flow.append(('Tampered request data', tampered_request_data))

        validation_result = validate_signed_request_data(
            data=tampered_request_data,
            secret_key=self.secret_key,
            signature_cls=signature_cls
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    def test_03_sign_url_and_validate_signed_req_data_tamper_extra_keys_add(
            self):
        """Fail tests for `sign_url` and `validate_signed_request_data`.

        As well as providing the additional data ``extra`` and data tampering
        `extra` keys (add).
        """
        flow = []
        for signature_cls in self.signature_classes:
            flow += \
                self.__t_sgn_url_and_vldt_sgnd_req_data_tamper_extra_keys_add(
                    signature_cls=signature_cls
                )
        return flow

    @log_info
    def __t_sgn_url_and_vldt_sgnd_req_data_tamper_extra_keys_add(
            self, signature_cls=Signature):
        """Tests for `sign_url` and `validate_signed_request_data`.

        As well as providing the additional data `extra` and data
        tampering `extra` keys (add) repeated params.
        """
        flow = []

        flow.append(('Signature class', signature_cls))

        signed_url = "{0}&provider=cervice0.example.com".format(
            self.__get_signed_url(signature_cls=signature_cls)
        )

        # *******************************************************************
        # ************************* Tampering *******************************
        # *******************************************************************

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        tampered_request_data = parse_url_params(signed_url)

        # *******************************************************************
        # ************************** Tampering ******************************
        # *******************************************************************
        even_more_tampered_request_data = copy(tampered_request_data)
        even_more_tampered_request_data['extra'] += ',provider'

        flow.append(('Tampered request data', tampered_request_data))
        flow.append(('Even more tampered request data',
                     even_more_tampered_request_data))

        validation_result = validate_signed_request_data(
            data=even_more_tampered_request_data,
            secret_key=self.secret_key,
            signature_cls=signature_cls
        )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    def test_04_sgn_url_vldt_signed_request_data_tamper_extra_keys_add(self):
        """Tests for `sign_url` and `validate_signed_request_data`.

        As well as providing the additional data `extra` and data tampering
        `extra` keys (add) repeated params.
        """
        flow = []
        for signature_cls in self.signature_classes:
            flow += \
                self.__t_sgn_url_and_vldt_sgnd_req_data_tamper_extra_keys_add(
                    signature_cls=signature_cls
                )
        return flow


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import absolute_import
import logging

from constance import config

from .base import BaseSkaAuthenticationBackend

logger = logging.getLogger(__file__)

__title__ = 'ska.contrib.django.ska.backends.constance_backend'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SkaAuthenticationConstanceBackend',)


class SkaAuthenticationConstanceBackend(BaseSkaAuthenticationBackend):
    """Authentication backend."""

    def get_settings(self):
        """

        :return:
        """
        return config.SKA_PROVIDERS



import datetime
import time

from base64 import b64encode

from six import text_type, python_2_unicode_compatible

from . import error_codes
from .defaults import SIGNATURE_LIFETIME, TIMESTAMP_FORMAT
from .helpers import sorted_urlencode

__title__ = 'ska.base'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SignatureValidationResult',
    'AbstractSignature',
)

# ****************************************************************************
# ****************************************************************************
# ******************************* Signature **********************************
# ****************************************************************************
# ****************************************************************************


@python_2_unicode_compatible
class SignatureValidationResult(object):
    """
    Signature validation result container.

    If signature validation result is True, things like this would work:

    >>> res = SignatureValidationResult(result=True)
    >>> print bool(res)
    True
    >>> res = SignatureValidationResult(
    >>>     result=False,
    >>>     reason=[error_codes.INVALID_SIGNATURE,]
    >>> )
    >>> print bool(res)
    False
    """

    # __slots__ = ('result', 'reason', 'errors')

    def __init__(self, result, errors=None):
        """Constructor."""
        self.result = result
        self.errors = errors if errors else {}

    def __str__(self):
        return str(self.result)
    __repr__ = __str__

    def __bool__(self):
        return self.result
    __nonzero__ = __bool__

    @property
    def message(self):
        """Human readable message of all errors.

        :return string:
        """
        return ' '.join(map(text_type, self.errors))

    @property
    def reason(self):
        """Reason.

        For backwards compatibility. Returns list of text messages.

        :return list:
        """
        return map(text_type, self.errors)


@python_2_unicode_compatible
class AbstractSignature(object):
    """Abstract class for signature generation and validation.

    Based on symmetric keys.

    :param str signature:
    :param str auth_user:
    :param float|str valid_until:
    """

    __slots__ = ('signature', 'auth_user', 'valid_until', 'extra')

    def __init__(self, signature, auth_user, valid_until, extra=None):
        """Constructor."""
        self.signature = signature
        self.auth_user = auth_user
        self.valid_until = valid_until
        self.extra = extra if extra else {}

    def __str__(self):
        return self.signature
    __repr__ = __str__

    def __bool__(self):
        return not self.is_expired()
    __nonzero__ = __bool__

    @classmethod
    def validate_signature(cls, signature, auth_user, secret_key, valid_until,
                           extra=None, return_object=False):
        """Validates the signature.

        :param str signature:
        :param str auth_user:
        :param str secret_key:
        :param float|str valid_until: Unix timestamp.
        :param dict extra: Extra arguments to be validated.
        :param bool return_object: If set to True, an instance of
            ``SignatureValidationResult`` is returned.
        :return bool:

        :example:
        >>> Signature.validate_signature(
        >>>     'EBS6ipiqRLa6TY5vxIvZU30FpnM=',
        >>>     'user',
        >>>     'your-secret-key',
        >>>     '1377997396.0'
        >>> )
        False
        """

        if isinstance(signature, str):
            signature = signature.encode()

        if not extra:
            extra = {}

        sig = cls.generate_signature(
            auth_user=auth_user,
            secret_key=secret_key,
            valid_until=valid_until,
            extra=extra
        )

        if not return_object:
            return sig.signature == signature and not sig.is_expired()

        else:
            result = (sig.signature == signature and not sig.is_expired())
            errors = []
            if sig.signature != signature:
                errors.append(error_codes.INVALID_SIGNATURE)
            if sig.is_expired():
                errors.append(error_codes.SIGNATURE_TIMESTAMP_EXPIRED)

            return SignatureValidationResult(result, errors)

    def is_expired(self):
        """Checks if current signature is expired.

        Returns True if signature is expired and False otherwise.

        :return bool:

        :example:
        >>> # Generating the signature
        >>> sig = Signature.generate_signature('user', 'your-secret-key')
        >>> sig.is_expired()
        False
        """
        now = datetime.datetime.now()
        valid_util = self.__class__.unix_timestamp_to_date(self.valid_until)

        # Expires > now is a valid condition here.
        res = valid_util > now

        # But we actually check agains is expired, so it's the opposite.
        return not res

    @classmethod
    def get_base(cls, auth_user, timestamp, extra=None):
        """Get base string.

        Add something here so that timestamp to signature conversion is not
        that obvious.

        :param string auth_user:
        :param int timestamp:
        :param dict extra:
        """
        if not extra:
            extra = {}

        _base = [str(timestamp), auth_user]

        if extra:
            urlencoded_extra = sorted_urlencode(extra)
            if urlencoded_extra:
                _base.append(urlencoded_extra)

        return ("_".join(_base)).encode()

    @staticmethod
    def make_secret_key(secret_key):
        """The secret key how its' supposed to be used in generate signature.

        :param str secret_key:
        :return str:
        """
        return secret_key.encode()  # return b64encode(secret_key)

    @classmethod
    def make_hash(cls, auth_user, secret_key, valid_until=None, extra=None):
        """Make hash.

        You should implement this method in your signature class.

        :param str auth_user:
        :param str secret_key:
        :param float|str valid_until: Unix timestamp, valid until.
        :param dict extra: Additional variables to be added.
        :return str:
        """
        raise NotImplementedError("You should implement this method!")

    @classmethod
    def generate_signature(cls, auth_user, secret_key, valid_until=None,
                           lifetime=SIGNATURE_LIFETIME, extra=None):
        """Generates the signature.

        If timestamp is given, the signature is created using the given
        timestamp. Otherwise current time is used.

        :param str auth_user:
        :param str secret_key:
        :param float|str valid_until: Unix timestamp, valid until.
        :param int lifetime: Lifetime of the signature in seconds.
        :param dict extra: Additional variables to be added.
        :return str:

        :example:
        >>> sig = Signature.generate_signature('user', 'your-secret-key')
        EBS6ipiqRLa6TY5vxIvZU30FpnM=
        """
        if not extra:
            extra = {}

        if not valid_until:
            valid_until = time.mktime(
                (
                    datetime.datetime.now() +
                    datetime.timedelta(seconds=lifetime)
                ).timetuple()
            )
        else:
            try:
                cls.unix_timestamp_to_date(valid_until)
            except Exception:
                return None  # Something went wrong

        signature = b64encode(
            cls.make_hash(auth_user, secret_key, valid_until, extra)
        )

        return cls(signature=signature, auth_user=auth_user,
                   valid_until=valid_until, extra=extra)

    @staticmethod
    def datetime_to_timestamp(dtv):
        """Human readable datetime according to the format specified.

         Format is specified in ``TIMESTAMP_FORMAT``.

        :param datetime.datetime dtv:
        :return str:
        """
        try:
            return dtv.strftime(TIMESTAMP_FORMAT)
        except Exception:
            pass

    @staticmethod
    def datetime_to_unix_timestamp(dtv):
        """Convert ``datetime.datetime`` to Unix timestamp.

        :param datetime.datetime dtv:
        :return float: Unix timestamp.
        """
        try:
            return time.mktime(dtv.timetuple())
        except Exception:
            pass

    @classmethod
    def timestamp_to_date(cls, timestamp, fail_silently=True):
        """Converts the given timestamp to date.

        If ``fail_silently`` is set to False, raises exceptions if timestamp
        is not valid timestamp (according to the format we have specified in
        the ``TIMESTAMP_FORMAT``). Mainly used internally.

        :param str timestamp:
        :param bool fail_silently:
        :return str:
        """
        try:
            return datetime.datetime.strptime(timestamp, TIMESTAMP_FORMAT)
        except Exception as err:
            if fail_silently is not True:
                raise err
            else:
                return None

    @classmethod
    def unix_timestamp_to_date(cls, timestamp, fail_silently=True):
        """Converts the given Unix timestamp to date.
        If ``fail_silently`` is set to False, raises exceptions if timestamp
        is not valid timestamp.

        :param float|str timestamp: UNIX timestamp. Possible to parse to float.
        :param bool fail_silently:
        :return str:
        """
        try:
            return datetime.datetime.fromtimestamp(float(timestamp))
        except Exception as err:
            if fail_silently is not True:
                raise err
            else:
                return None



from .defaults import (
    SIGNATURE_LIFETIME,
    DEFAULT_URL_SUFFIX,
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_EXTRA_PARAM
)
from .signatures import Signature
from .utils import RequestHelper


__title__ = 'ska.shortcuts'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'extract_signed_request_data',
    'sign_url',
    'signature_to_dict',
    'validate_signed_request_data',
)


# **************************************************************************
# **************************************************************************
# ************************** Shortcut functions ****************************
# **************************************************************************
# **************************************************************************


def sign_url(auth_user,
             secret_key,
             valid_until=None,
             lifetime=SIGNATURE_LIFETIME,
             url='',
             suffix=DEFAULT_URL_SUFFIX,
             signature_param=DEFAULT_SIGNATURE_PARAM,
             auth_user_param=DEFAULT_AUTH_USER_PARAM,
             valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
             extra=None,
             extra_param=DEFAULT_EXTRA_PARAM,
             signature_cls=Signature):
    """Sign the URL.

    :param str auth_user: Username of the user making the request.
    :param str secret_key: The shared secret key.
    :param float|str valid_until: Unix timestamp. If not given, generated
        automatically (now + lifetime).
    :param int lifetime: Signature lifetime in seconds.
    :param str url: URL to be signed.
    :param str suffix: Suffix to add after the ``endpoint_url`` and before
        the appended signature params.
    :param str signature_param: Name of the GET param name which would hold
        the generated signature value.
    :param str auth_user_param: Name of the GET param name which would hold
        the ``auth_user`` value.
    :param str valid_until_param: Name of the GET param name which would
        hold the ``valid_until`` value.
    :param dict extra: Extra variables to add to the request.
    :param str extra_param: Name of the GET param name which would hold the
        ``extra_keys`` value.
    :param signature_cls:
    :return str:

    :example:
    Required imports.

    >>> from ska import sign_url

    Producing a signed URL.

    >>> signed_url = sign_url(
    >>>     auth_user='user', secret_key='your-secret_key', lifetime=120,
    >>>     url='http://e.com/api/', signature_param=DEFAULT_SIGNATURE_PARAM,
    >>>     auth_user_param=DEFAULT_AUTH_USER_PARAM,
    >>>     valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
    >>>     extra={
    >>>         'provider': 'service1.example.com',
    >>>         'email': 'john.doe@mail.example.com'
    >>>     },
    >>>     extra_param = DEFAULT_EXTRA_PARAM
    >>> )
    http://e.com/api/?valid_until=1378045287.0&auth_user=user&signature=
    YlZpLFsjUKBalL4x5trhkeEgqE8%3D
    """
    if lifetime is None:
        lifetime = SIGNATURE_LIFETIME

    if not extra:
        extra = {}

    assert isinstance(lifetime, int)

    signature = signature_cls.generate_signature(
        auth_user=auth_user,
        secret_key=secret_key,
        valid_until=valid_until,
        lifetime=lifetime,
        extra=extra
    )

    request_helper = RequestHelper(
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra_param=extra_param,
        signature_cls=signature_cls
    )

    signed_url = request_helper.signature_to_url(
        signature=signature,
        endpoint_url=url,
        suffix=suffix,
    )

    return signed_url


def signature_to_dict(auth_user,
                      secret_key,
                      valid_until=None,
                      lifetime=SIGNATURE_LIFETIME,
                      signature_param=DEFAULT_SIGNATURE_PARAM,
                      auth_user_param=DEFAULT_AUTH_USER_PARAM,
                      valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                      extra=None,
                      extra_param=DEFAULT_EXTRA_PARAM,
                      signature_cls=Signature):
    """Return a dictionary containing the signature data params.

    :param str auth_user: Username of the user making the request.
    :param str secret_key: The shared secret key.
    :param float|str valid_until: Unix timestamp. If not given, generated
        automatically (now + lifetime).
    :param int lifetime: Signature lifetime in seconds.
    :param str signature_param: Name of the (for example POST) param name
        which would hold the generated ``signature`` value.
    :param str auth_user_param: Name of the (for example POST) param name
        which would hold the ``auth_user`` value.
    :param str valid_until_param: Name of the (for example POST) param name
        which would hold the ``valid_until`` value.
    :param dict extra: Additional arguments for the signature.
    :param str extra_param: Name of the (for example POST) param name which
        would hold the ``extra`` keys value.
    :param signature_cls:
    :return str:

    :example:
    Required imports.

    >>> from ska import signature_to_dict

    Producing a dictionary with signature data.

    >>> signature_dict = signature_to_dict(
    >>>     auth_user='user', secret_key='your-secret_key', lifetime=120,
    >>>     signature_param=DEFAULT_SIGNATURE_PARAM,
    >>>     auth_user_param=DEFAULT_AUTH_USER_PARAM,
    >>>     valid_until_param=DEFAULT_VALID_UNTIL_PARAM
    >>> )
    {
        'signature': 'YlZpLFsjUKBalL4x5trhkeEgqE8=',
        'auth_user': 'user',
        'valid_until': '1378045287.0'
    }
    """
    if lifetime is None:
        lifetime = SIGNATURE_LIFETIME

    if not extra:
        extra = {}

    assert isinstance(lifetime, int)

    signature = signature_cls.generate_signature(
        auth_user=auth_user,
        secret_key=secret_key,
        valid_until=valid_until,
        lifetime=lifetime,
        extra=extra
    )

    request_helper = RequestHelper(
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra_param=extra_param,
        signature_cls=signature_cls
    )

    signature_dict = request_helper.signature_to_dict(
        signature=signature
    )

    return signature_dict


def validate_signed_request_data(data, secret_key,
                                 signature_param=DEFAULT_SIGNATURE_PARAM,
                                 auth_user_param=DEFAULT_AUTH_USER_PARAM,
                                 valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                                 extra_param=DEFAULT_EXTRA_PARAM,
                                 signature_cls=Signature):
    """Validate the signed request data.

    :param dict data: Dictionary holding the (HTTP) request (for example GET
        or POST) data.
    :param str secret_key: The shared secret key.
    :param str signature_param: Name of the (for example GET or POST) param
        name which holds the ``signature`` value.
    :param str auth_user_param: Name of the (for example GET or POST) param
        name which holds the ``auth_user`` value.
    :param str valid_until_param: Name of the (foe example GET or POST)
        param name which holds the ``valid_until`` value.
    :param str extra_param: Name of the (foe example GET or POST) param
        name which holds the ``extra`` keys value.
    :param signature_cls:
    :return ska.SignatureValidationResult: A ``ska.SignatureValidationResult``
        object with the following properties:
            - `result` (bool): True if data is valid. False otherwise.
            - `reason` (list): List of strings, indicating validation errors.
              Empty list in case if `result` is True.
    """
    request_helper = RequestHelper(
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra_param=extra_param,
        signature_cls=signature_cls
    )

    validation_result = request_helper.validate_request_data(
        data=data,
        secret_key=secret_key
    )

    return validation_result


def extract_signed_request_data(data, secret_key=None,
                                signature_param=DEFAULT_SIGNATURE_PARAM,
                                auth_user_param=DEFAULT_AUTH_USER_PARAM,
                                valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                                extra_param=DEFAULT_EXTRA_PARAM,
                                validate=False, fail_silently=False,
                                signature_cls=Signature):
    """Validate the signed request data.

    :param dict data: Dictionary holding the (HTTP) request (for example
        GET or POST) data.
    :param str secret_key: The shared secret key.
    :param str signature_param: Name of the (for example GET or POST) param
        name which holds the ``signature`` value.
    :param str auth_user_param: Name of the (for example GET or POST) param
        name which holds the ``auth_user`` value.
    :param str valid_until_param: Name of the (foe example GET or POST) param
        name which holds the ``valid_until`` value.
    :param str extra_param: Name of the (foe example GET or POST) param name
        which holds the ``extra`` value.
    :param bool validate: If set to True, request data is validated before
        returning the result.
    :param bool fail_silently: If set to True, exceptions are omitted.
    :param signature_cls:
    :return dict: Dictionary with signed request data.
    """
    request_helper = RequestHelper(
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra_param=extra_param,
        signature_cls=signature_cls
    )

    return request_helper.extract_signed_data(
        data,
        secret_key=secret_key,
        validate=validate,
        fail_silently=fail_silently
    )



from __future__ import absolute_import, print_function

import logging
import os
import random

import pytest

__title__ = 'ska.contrib.django.ska.tests.helpers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'


logger = logging.getLogger(__name__)


def project_dir(base):
    """Project dir."""
    return os.path.join(os.path.dirname(__file__), base).replace('\\', '/')


PROJECT_DIR = project_dir

LOG_INFO = True


def log_info(func):
    """Logs some useful info."""
    if not LOG_INFO:
        return func

    def inner(self, *args, **kwargs):
        """Inner."""
        result = func(self, *args, **kwargs)

        logger.debug('\n\n%s', func.__name__)
        logger.debug('============================')
        if func.__doc__:
            logger.debug('""" %s """', func.__doc__.strip())
        logger.debug('----------------------------')
        if result is not None:
            logger.debug(result)
        logger.debug('\n++++++++++++++++++++++++++++')

        return result
    return inner


def change_date():
    """Change date."""
    return bool(random.randint(0, 1))


NUM_ITEMS = 5

# *********************************************************************
# *********************************************************************
# *********************************************************************

# Skipping from non-Django tests.
if os.environ.get("DJANGO_SETTINGS_MODULE", None):

    import factories

    SKA_TEST_USER_USERNAME = factories.TEST_ADMIN_USERNAME
    SKA_TEST_USER_PASSWORD = factories.TEST_PASSWORD

# *********************************************************************
# *********************************************************************
# *********************************************************************

    @pytest.mark.django_db
    def create_admin_user():
        """Create a user for testing the dashboard.

        TODO: At the moment an admin account is being tested. Automated tests
        with diverse accounts are to be implemented.
        """
        user = factories.TestAdminUsernameSuperAdminUserFactory(
            email='admin@dev.django-ska.com'
        )

    @pytest.mark.django_db
    def generate_data(num_items=NUM_ITEMS):
        """Generate data."""
        return factories.FooItemFactory.create_batch(num_items)



import os
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

version = '1.6.11'

exec_dirs = [
    'src/ska/bin/',
]

execs = []
for exec_dir in exec_dirs:
    execs += [os.path.join(exec_dir, f) for f in os.listdir(exec_dir)]

setup(
    name='ska',
    version=version,
    description="Sign- and validate- data (dictionaries, strings) using "
                "symmetric-key algorithm.",
    long_description=readme,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Security :: Cryptography",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or "
        "later (LGPLv2+)",
    ],
    keywords='sign data, sign (HTTP) request, symmetric-key algorithm '
             'encryption, sign URL, python, django, password-less login '
             'django, password-less authentication backend django',
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    url='https://github.com/barseghyanartur/ska',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    include_package_data=True,
    package_data={
        'ska': execs,
    },
    scripts=['src/ska/bin/ska-sign-url'],
    install_requires=[
        'six>=1.9',
        'django-nine>=0.1.10',
    ],
    extras_require={
        'django-constance': ['django-constance'],
    },
    tests_require=[
        'factory_boy',
        'fake-factory',
        'pytest',
        'pytest-django',
        'radar',
    ]
)
